/*
 * Copyright 2010 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

///////////////////////////////////////////////////////////////////////////////
// PROJECT:  C++TESK
// AUTHOR:   Alexander Kamkin (kamkin@ispras.ru)
// SYNOPSIS: Set of reactions 
///////////////////////////////////////////////////////////////////////////////

#include <hw/reactions.hpp>

#include <sstream>

#include <assert.h>
#include <string.h>

#ifdef CPPTESK_BUILD_WITH_OLD_ASPECTRACE_PATH
extern "C" {
#include <aspectrace.h>
#include <aspect_error.h>
}
#else
extern "C" {
#include <aspectrace/aspectrace.h>
#include <aspectrace/aspect_coverage.h>
}
#endif //CPPTESK_BUILD_WITH_OLD_ASPECTRACE_PATH

namespace cpptesk {
namespace hw {

Reactions::Reactions()
{
    reset();
}

Reactions::~Reactions()
{
}

void Reactions::registerReaction(const Reaction &reaction)
{
    _reactions.push_back(reaction);
    _counts[reaction.getError()]++;
}

void Reactions::clearReactions()
{
    // Append the current set to the history
    for(reactions_list_t::iterator i = _reactions.begin(); i != _reactions.end(); i++)
        { _history.push_back(*i); }

    // Reduce the history
    if(_history.size() > HISTORY_SIZE)
        { _history.erase(_history.begin(), _history.begin() + (_history.size() - HISTORY_SIZE)); }

    // Clear the current set
    _reactions.clear();
}

const std::vector<Reaction>& Reactions::getReactions() const
{
    return _reactions;
}

bool Reactions::verdict()
{
    bool result = true;

    // If all reactions are normal, then the verdict is true
    for(reactions_list_t::iterator i = _reactions.begin(); i != _reactions.end(); i++)
    {
        // Otherwise, the verdict is false
        if(i->getError() != NORMAL_REACTION)
            { 
		result = false; 
		aspect_error_traceFailure("error", i->getShortDescription().c_str(), i->getFullDescription().c_str()); 
	    }
    }

    // The reactions have been analyzed and there is no reason to analyze them once again
    clearReactions();

    return result;
}

std::string Reactions::error(error_mode_t mode) const
{
    bool written  = false;
    bool finished = false;
    std::stringstream buffer;

    // Analyze the current set of reactions
    for(reactions_list_t::const_reverse_iterator i = _reactions.rbegin(); i != _reactions.rend(); i++)
    {
        if(i->getError() != NORMAL_REACTION)
        {
            if(written) { buffer << std::endl; }
            buffer << i->getFullDescription();
            if(mode == LAST_ERROR) { finished = true; break; }
            written = true;
        }

        if(mode == LAST_REACTION) { finished = true; break; }
    }

    if(finished)
        { return buffer.str(); }

    // Analyze the history of reactions
    for(reactions_list_t::const_reverse_iterator i = _history.rbegin(); i != _history.rend(); i++)
    {
        if(i->getError() != NORMAL_REACTION)
        {
            if(written) { buffer << std::endl; }
            buffer << i->getFullDescription();
            if(mode == LAST_ERROR) { break; }
            written = true;
        }

        if(mode == LAST_REACTION) { break; }
    }

    return buffer.str();
}

std::string Reactions::diagnosis() const
{
    // TODO: Analyze the history only
    return "";
}

unsigned Reactions::count(error_type_t error) const
{
    return _counts[error];
}

void Reactions::reset()
{
    // Reset the current set of reactions
    _reactions.clear();
    // Reset the history of reactions
    _history.clear();
    // Reset statistics of reactions
    ::memset(_counts, 0, sizeof(_counts));
}

}} // namespace cpptesk::hw

