/** @file fifo_media.cpp
 * This file contains the mediator class implementation.
 *
 * @author Mikhail Chupilko (chupilko@ispras.ru)
 */

#include <fifo_media.h>

#include <sync.h>

#include <interface.h>
#include <vpi_media.h>

#include <iostream>

namespace cpptesk {
namespace examples {
namespace fifo {

FIFOMediator::FIFOMediator()
{
    // Registration of interfaces, serializers, and deserializers
    SET_INPUT_ADAPTER(iface1, FIFOMediator::serialize_iface1);
    SET_INPUT_ADAPTER(iface2, FIFOMediator::serialize_iface2);
    SET_OUTPUT_ADAPTER(iface3, FIFOMediator::deserialize_iface3);

    // Maximum reaction waiting time
    SET_REACTION_TIMEOUT(FIFOMediator::REACTION_TIMEOUT);

    // Registration of the iface3's arbiter
    SET_ARBITER(iface3, matcher_iface3);

    // Start detecting unexpected reactions on iface3
    CALL_OUTPUT_LISTENER(FIFOMediator::listen_iface3, iface3);

    // Set level of debug messages
    debug(DEBUG_ERROR);
}

FIFOMediator::~FIFOMediator() {}

bool FIFOMediator::isIface1Ready() const
{
    return FIFO::isIface1Ready();
}
bool FIFOMediator::isIface2Ready() const
{
    return FIFO::isIface2Ready();
}

DEFINE_PROCESS(FIFOMediator::serialize_iface1)
{
    // Copy message to the local variable
    InputData msg = CAST_MESSAGE(InputData);

    // Start the serializer's working
    START_PROCESS();

#ifdef DEBUG
	std::cout << "FIFOMediator::serialize_iface1" << std::endl;
#endif

    // Set iface1 state to busy
    CAPTURE_IFACE();

    // Set values to the DUT's inputs
    inputs.CE_WR = 1;
    inputs.DI = msg.get_data();

    // Set iface1 state to free
    RELEASE_IFACE();

    // The process is stopped until the next cycle of simulation
    CYCLE();

    // Stop the serializer's working
    STOP_PROCESS();
}

DEFINE_PROCESS(FIFOMediator::serialize_iface2)
{
    // Copy message to the local variable
    InputData msg = CAST_MESSAGE(InputData);

    // Start the serializer's working
    START_PROCESS();

#ifdef DEBUG
	std::cout << "FIFOMediator::serialize_iface2" << std::endl;
#endif

    // Set iface2 state to busy
    CAPTURE_IFACE();

    // Set values to the DUT's inputs
    inputs.CE_RD = 1;

    // Set iface2 state to free
    RELEASE_IFACE();

    // The process is stopped until the next cycle of simulation
    CYCLE();

    // Stop the serializer's working
    STOP_PROCESS();
}

DEFINE_PROCESS(FIFOMediator::deserialize_iface3)
{
    // Copy message to the local variable
    OutputData &msg = CAST_MESSAGE(OutputData);

    // Start the deserializer's working
    START_PROCESS();

#ifdef DEBUG
        std::cout << "FIFOMediator::deserialize_iface3" << std::endl;
#endif

    // Wait several cycles until DUT sets DO_VAL==1
    WAIT_REACTION(outputs.DO_VAL);

    // Save data sent by DUT
    msg.set_data(outputs.DO);

    // Select next registered reaction at this interface if given
    NEXT_REACTION();

    // Stop the deserializer's working
    STOP_PROCESS();
}

}}}

